﻿using System;
using System.Collections.Generic;
using System.Linq.Expressions;
using System.Reflection;
using MugenInjection.Delegates;
using MugenInjection.Infrastructure;

namespace MugenInjection.Activators
{
    /// <summary>
    ///  Represents the activator that use <see cref="Expression"/> to activate object.
    /// </summary>
    public class ExpressionActivator
#if FULLEXPRESSION
 : ActivatorBase
#else
 : EmitActivator
#endif

    {

        #region Fields

        private readonly CacheCollection<ConstructorInfo, ActivatorDelegate> _constructorActivators =
            new CacheCollection<ConstructorInfo, ActivatorDelegate>(3);
#if FULLEXPRESSION
        private readonly CacheCollection<MemberInfo, Delegate> _methodActivators =
            new CacheCollection<MemberInfo, Delegate>(5);

        private readonly CacheCollection<MemberInfo, Delegate> _propertyActivators =
            new CacheCollection<MemberInfo, Delegate>(5);
#endif

        #endregion

        #region Overrides of ActivatorBase

        /// <summary>
        ///     Creates a new instance of the specified service.
        /// </summary>
        /// <param name="constructorInfo">
        ///     The specified <see cref="ConstructorInfo" />.
        /// </param>
        protected override object Activate(ConstructorInfo constructorInfo)
        {
            ActivatorDelegate constructorActivator = _constructorActivators[constructorInfo];
            if (constructorActivator == null)
            {
#if FULLEXPRESSION
                constructorActivator = ExpressionReflectionAccessProvider.GetActivatorDelegate(constructorInfo, false);
#else
                ParameterExpression parameterExpression;
                Expression[] expressions = GetParametersExpression(constructorInfo, out parameterExpression);
                Expression newExpression = Expression.New(constructorInfo, expressions);
                if (constructorInfo.DeclaringType.IsValueType)
                    newExpression = Expression.Convert(newExpression, typeof(object));
                constructorActivator = Expression.Lambda<ActivatorDelegate>(newExpression, parameterExpression).Compile();
#endif
                _constructorActivators[constructorInfo] = constructorActivator;
            }
            return constructorActivator.Invoke(CurrentContext.Injector
                                                             .Components.BindingActivatorComponent
                                                             .GetParameters(constructorInfo, CurrentContext));
        }

#if FULLEXPRESSION


        /// <summary>
        ///     Injects values into properties, fields, methods in the specified object.
        /// </summary>
        /// <param name="targetType">
        ///     The specified target <see cref="Type" />.
        /// </param>
        /// <param name="target">The specified target for inject.</param>
        protected override void Inject<T>(Type targetType, ref T target)
        {
            Type attributeForInject = CurrentContext.Injector.Settings.AttributeForInject;
            if (!IsHasValuesToInject(targetType, attributeForInject)) return;
            var activatorComponent = CurrentContext.Injector.Components.BindingActivatorComponent;

            IList<PropertyInfo> propertiesToInject = GetPropertiesToInject(targetType, attributeForInject);
            if (propertiesToInject.Count > 0)
            {
                for (int i = 0; i < propertiesToInject.Count; i++)
                {
                    PropertyInfo propertyInfo = propertiesToInject[i];
                    var propertyActivator = _propertyActivators[propertyInfo] as SetterValueTypeDelegate<T>;
                    if (propertyActivator == null)
                    {
                        propertyActivator = ExpressionReflectionAccessProvider
                            .GetAssignMemberDelegate<SetterValueTypeDelegate<T>, T>(
                                propertyInfo, true, false);
                        _propertyActivators[propertyInfo] = propertyActivator;
                    }
                    propertyActivator.Invoke(ref target, activatorComponent.GetParameter(propertyInfo, CurrentContext));
                }
            }


            IList<MethodInfo> methodsToInject = GetMethodsToInject(targetType, attributeForInject);
            if (methodsToInject.Count > 0)
            {
                for (int i = 0; i < methodsToInject.Count; i++)
                {
                    MethodInfo methodInfo = methodsToInject[i];
                    var methodActivator = _methodActivators[methodInfo] as InvokeMethodValueTypeDelegate<T>;
                    if (methodActivator == null)
                    {
                        methodActivator =
                            ExpressionReflectionAccessProvider
                                .GetInvokeMethodDelegate<InvokeMethodValueTypeDelegate<T>, T>(methodInfo, true, false);
                        _methodActivators[methodInfo] = methodActivator;
                    }
                    methodActivator.Invoke(ref target, activatorComponent.GetParameters(methodInfo, CurrentContext));
                }
            }
        }
#endif

        /// <summary>
        /// Releases resources held by the object.
        /// </summary>
        protected override void Dispose(bool disposing)
        {
            if (disposing && !IsDisposed)
            {
                _constructorActivators.Clear();
#if FULLEXPRESSION
                _propertyActivators.Clear();
                _methodActivators.Clear();
#endif
            }
            base.Dispose(disposing);
        }

        #endregion

        #region Methods

#if !FULLEXPRESSION
        private static Expression[] GetParametersExpression(MethodBase methodBase, out ParameterExpression parameterExpression)
        {
            ParameterInfo[] paramsInfo = methodBase.GetParameters();
            //create a single param of type object[]
            parameterExpression = Expression.Parameter(typeof(object[]), "args");
            var argsExp = new Expression[paramsInfo.Length];

            //pick each arg from the params array 
            //and create a typed expression of them
            for (int i = 0; i < paramsInfo.Length; i++)
            {
                Expression index = Expression.Constant(i);
                Type paramType = paramsInfo[i].ParameterType;
                Expression paramAccessorExp = Expression.ArrayIndex(parameterExpression, index);
                Expression paramCastExp = Expression.Convert(paramAccessorExp, paramType);
                argsExp[i] = paramCastExp;
            }
            return argsExp;
        }
#endif

        #endregion
    }
}