﻿using System;
using System.Collections.Generic;
using MugenInjection.Delegates;
using MugenInjection.Infrastructure;
using MugenInjection.Interface;

namespace MugenInjection.Bindings
{
    /// <summary>
    /// Represents the custom binding.
    /// </summary>
    public class CustomBinding : BindingBase
    {
        #region Fields

        /// <summary>
        /// Gets the <see cref="IActivator"/> for activate object.
        /// </summary>
        private readonly IActivator _activator;

        #endregion

        #region Constructor

        /// <summary>
        /// Initializes a new instance of the <see cref="CustomBinding" /> class.
        /// </summary>
        /// <param name="customBinding">The specified custom binding.</param>
        /// <param name="activator">The specified <see cref="IActivator"/> for activate object. </param>
        /// <param name="services">The specified services types.</param>
        /// <param name="scopeLifecycle">The specified <see cref="IScopeLifecycle"/></param>
        /// <param name="parameters">The specified parameters.</param>
        /// <param name="canResolve">The specified condition for can resolve method.</param>
        public CustomBinding(IBinding customBinding, IActivator activator, IList<Type> services,
                             IScopeLifecycle scopeLifecycle,
                             IEnumerable<IInjectionParameter> parameters, CanResolveBindingDelegate canResolve)
            : base(services, scopeLifecycle, parameters, canResolve)
        {
            Binding = customBinding;
            Validate.ArgumentNotNull(customBinding, "customBinding");

            _activator = activator;

            ActivatedActions.Add(InjectorUtilsInternal.ActivateInitializable);
            ActivatedActions.Add(InjectorUtilsInternal.ActivateStartable);
            if (scopeLifecycle != null && scopeLifecycle.ReleaseObjectActions != null &&
                !scopeLifecycle.ReleaseObjectActions.IsReadOnly)
                scopeLifecycle.ReleaseObjectActions.Add(InjectorUtilsInternal.DeactivateStartable);
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets <see cref="IActivator"/> for the current binding.
        /// </summary>
        public IActivator Activator
        {
            get { return _activator; }
        }

        /// <summary>
        /// Gets <see cref="IBinding"/> to activate object.
        /// </summary>
        public IBinding Binding { get; set; }

        #endregion

        #region Overrides of BindingBase

        /// <summary>
        /// Determines whether the specified request can be resolved.
        /// </summary>
        /// <param name="bindingContext">The specified <see cref="IBindingContext"/>.</param>
        /// <returns><c>True</c> if the specified service has been resolved; otherwise, <c>false</c>.</returns>
        public override bool CanResolve(IBindingContext bindingContext)
        {
            return Binding.CanResolve(bindingContext);
        }

        /// <summary>
        /// Resolves instance for the specified <see cref="IBindingContext"/>.
        /// </summary>
        /// <param name="bindingContext">The specified <see cref="IBindingContext"/>.</param>
        /// <returns>An instance of the service.</returns>
        protected override object ResolveService(IBindingContext bindingContext)
        {
            object resolve = Binding.Resolve(bindingContext);
            if (_activator != null)
                _activator.Activate(ref resolve, bindingContext);
            return resolve;
        }

        /// <summary>
        /// Releases resources held by the object.
        /// </summary>
        protected override void Dispose(bool disposing)
        {
            if (disposing && !IsDisposed)
            {
                Binding.Dispose();
                if (_activator != null)
                    _activator.Dispose();
            }
            base.Dispose(disposing);
        }

        #endregion
    }
}